@@ -4,7 +4,7 @@ package Inline::CPP::Parser::RecDescent;
 # Dev versions will have a _0xx suffix.
 # We eval the $VERSION to accommodate dev version numbering as described in
 # perldoc perlmodstyle
-our $VERSION = '0.75';
+our $VERSION = '0.76';
 #$VERSION = eval $VERSION;  ## no critic (eval)
 
 use Carp;
@@ -103,6 +103,15 @@ sub grammar {
 { use Data::Dumper; }
 
 {
+    sub handle_args {
+        my ($args) = @_;
+        my %argsdef;
+        $argsdef{arg_names} = [ map $_->{name}, @$args ];
+        $argsdef{arg_types} = [ map $_->{type}, @$args ];
+        $argsdef{arg_offsets} = [ map $_->{offset}, @$args ];
+        $argsdef{arg_optional} = [ map $_->{optional}, @$args ];
+        \%argsdef;
+    }
     sub handle_class_def {
         my ($thisparser, $def) = @_;
 #         print "Found a class: $def->[0]\n";
@@ -169,7 +178,9 @@ part: comment
                           $item[1]->{args});
      push @{$thisparser->{data}{functions}}, $name
            unless defined $thisparser->{data}{function}{$name};
-     $thisparser->{data}{function}{$name} = $item[1];
+     my %funcdef = %{ $item[1] };
+     %funcdef = (%funcdef, %{ handle_args(delete $funcdef{args}) });
+     $thisparser->{data}{function}{$name} = \%funcdef;
 #    print Dumper $item[1];
      1;
       }
@@ -250,7 +261,9 @@ class_decl: comment { [{thing => 'comment'}] }
           }
           Inline::CPP::Parser::RecDescent::strip_ellipsis($thisparser,
                            $item[1]->{args});
-          [$item[1]];
+          my %funcdef = %{ $item[1] };
+          %funcdef = (%funcdef, %{ handle_args(delete $funcdef{args}) });
+          [\%funcdef];
         }
           | member_def
         {
@@ -269,15 +282,15 @@ function_def: operator <commit> ';'
               }
             | IDENTIFIER '(' <commit> <leftop: arg ',' arg>(s?) ')' smod(?) code_block
               {
-                {name => $item{IDENTIFIER}, args => $item{__DIRECTIVE2__}, rtype => '' }
+                {name => $item{IDENTIFIER}, args => $item{__DIRECTIVE2__}, return_type => '' }
               }
-            | rtype IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' ';'
+            | return_type IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' ';'
               {
-                {rtype => $item[1], name => $item[2], args => $item{__DIRECTIVE1__} }
+                {return_type => $item[1], name => $item[2], args => $item{__DIRECTIVE1__} }
               }
-            | rtype IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' smod(?) code_block
+            | return_type IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' smod(?) code_block
               {
-                {rtype => $item{rtype}, name => $item[2], args => $item{__DIRECTIVE1__} }
+                {return_type => $item{return_type}, name => $item[2], args => $item{__DIRECTIVE1__} }
               }
 
 method_def: operator <commit> method_imp
@@ -291,18 +304,18 @@ method_def: operator <commit> method_imp
 #         print "con-/de-structor found: $item[1]\n";
               {name => $item[1], args => $item{__DIRECTIVE2__}, abstract => ${$item{method_imp}} };
             }
-          | rtype IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' method_imp
+          | return_type IDENTIFIER '(' <leftop: arg ',' arg>(s?) ')' method_imp
             {
 #         print "method found: $item[2]\n";
           $return =
-                {name => $item[2], rtype => $item[1], args => $item[4],
+                {name => $item[2], return_type => $item[1], args => $item[4],
              abstract => ${$item[6]},
                  rconst => $thisparser->{data}{smod}{const},
                 };
           $thisparser->{data}{smod}{const} = 0;
             }
 
-operator: rtype(?) 'operator' /\(\)|[^()]+/ '(' <leftop: arg ',' arg>(s?) ')'
+operator: return_type(?) 'operator' /\(\)|[^()]+/ '(' <leftop: arg ',' arg>(s?) ')'
           {
 #            print "Found operator: $item[1][0] operator $item[3]\n";
             {name=> "operator $item[3]", args => $item[5], ret => $item[1][0]}
@@ -373,7 +386,7 @@ IDENTIFIER: <leftop: ident_part '::' ident_part>
 
 # Parse::RecDescent is retarded in this one case: if a subrule fails, it
 # gives up the entire rule. This is a stupid way to get around that.
-rtype: rtype2 | rtype1
+return_type: rtype2 | rtype1
 rtype1: TYPE star(s?)
         {
          $return = $item[1];
